package ru.pricepulse.parsingservice.ozon_parser.service.marketplace.ozon.parsing;

import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.support.ui.WebDriverWait;
import org.slf4j.MDC;
import org.springframework.retry.annotation.Recover;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Service;
import ru.pricepulse.parsingservice.ozon_parser.pool.WebDriverPool;
import ru.pricepulse.parsingservice.ozon_parser.service.marketplace.ozon.page.AccessDeniedPage;
import ru.pricepulse.parsingservice.ozon_parser.service.marketplace.ozon.page.CategoryPage;

import java.time.Duration;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class CategoryPageParsingService {

    private final ExecutorService productPageExecutor = Executors.newFixedThreadPool(3);

    private final WebDriverPool webDriverPool;

    public CategoryPageParsingService(WebDriverPool webDriverPool) {
        this.webDriverPool = webDriverPool;
    }

    @Retryable(maxAttempts = 10, recover = "recover")
    public void parseCategoryPage(int finalPageIndex, String url, ArrayList<String> errors) throws InterruptedException {
        MDC.put("pageIndex", String.valueOf(finalPageIndex));
        String pageUrl = url + "/?page=" + finalPageIndex;
        var driver = webDriverPool.borrowDriver();

        try {
            driver.manage().timeouts().pageLoadTimeout(Duration.of(10, ChronoUnit.SECONDS));
            driver.get(pageUrl);
            WebDriverWait wait = new WebDriverWait(driver, Duration.of(10, ChronoUnit.SECONDS));
            var accessDeniedPage = new AccessDeniedPage(driver, wait); // TODO подумать как не создавать кучу PageObject
            var categoryPage = new CategoryPage(driver, wait);
            wait.until(d -> checkForWaitingPageLoading(accessDeniedPage, categoryPage));
            if (checkAccessDeniedPage(accessDeniedPage)) {
                log.info("Доступ ограничен, пробуем решить проблему: {}", pageUrl);
                resolveAccessDeniedPage(accessDeniedPage);
                log.info("Проблема успешно решена: {}", pageUrl);
            }
            log.info("Получаем список ссылок на товары на текущей странице: {}", pageUrl);
            Set<String> hrefs = Set.of();
            try {
                hrefs = categoryPage.getProductsLinks();
            } catch (Exception e) {
                throw new Exception(e);
            }
            webDriverPool.returnDriver(driver);
            log.info("Страница {} Получены ссылки на товары: {}", finalPageIndex, hrefs.size());
            hrefs.forEach(href -> {
                MDC.put("pageIndex", String.valueOf(finalPageIndex));
                try {
                    processPage(href);
                    errors.add(href);
                    log.error(String.valueOf(errors.size()));
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            });
            /*hrefs.forEach(href -> productPageExecutor.submit(() -> {
                MDC.put("pageIndex", String.valueOf(finalPageIndex));
                try {
                    processPage(href);
                    errors.add(href);
                    log.error(String.valueOf(errors.size()));
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            }));*/
            productPageExecutor.awaitTermination(10, TimeUnit.SECONDS);
        } catch (Exception ignored) {
            throw new RuntimeException(ignored);
        } finally {
            webDriverPool.returnDriver(driver); // Завершаем работу драйвера
        }
    }

    private String processPage(String href) throws InterruptedException {
        var driver = webDriverPool.borrowDriver();
        try {
            driver.get(href);
            log.info("Страница обработана");
        } catch (Throwable ignored) {

        } finally {
            webDriverPool.returnDriver(driver); // Завершаем работу драйвера
        }
        return href;
    }

    private boolean checkForWaitingPageLoading(AccessDeniedPage accessDeniedPage,
                                               CategoryPage categoryPage) {
        log.debug("Проверка что страница 'Доступ ограничен'");
        try {
            if (checkAccessDeniedPage(accessDeniedPage)) {
                return true;
            }
        } catch (Exception e) {
            log.debug("Ошибка проверки", e);
        }
        log.debug("Проверка что страница 'Страница категории'");
        if (checkCategoryPage(categoryPage)) {
            return true;
        }
        log.debug("Проверка загрузки страницы неудачна");
        return false;
    }

    private boolean checkAccessDeniedPage(AccessDeniedPage accessDeniedPage) {
        return accessDeniedPage.isLoaded();
    }

    private boolean checkCategoryPage(CategoryPage categoryPage) {
        return categoryPage.isLoaded();
    }

    private void resolveAccessDeniedPage(AccessDeniedPage accessDeniedPage) {
        accessDeniedPage.clickReloadButton();
    }

    @Recover
    private void recover(Exception e) {
        log.error("Все ретраи провалились");
    }

}
